C64=0

	if C64=1
	org $0801-2
	dw  $0801
	else
	org $1001-2
	dw  $1001
	endif
	
	if C64=1
RASTER = $D012
REGNUM = $DF40
REGVAL = $DF50
STATUS = $DF60
	else
RASTER = $FF1D
REGNUM = $FDE4
REGVAL = $FDE5
STATUS = $FDE4
	endif

; VGM format specs:
; https://vgmrips.net/wiki/VGM_Specification

; VGM files:
; https://vgmrips.net/packs/pack/adlib-music-synthesizer-card-demo-songs-ibm-pc-xt-at

; To write value V to register R do the following:
; 1. Write R to $FDE2
; 2. Wait 3.3 microseconds
; 3. Write V to $FDE3
; 4. Wait 23 microseconds
;
; If you need to read the status, just read $FDE2. Everything should behave as an AdLib card for the PC, except that the addresses are different
; This document should thus be fully applicable if you're curious: https://bochs.sourceforge.io/techspec/adlib_sb.txt.
;
; At startup, write 0 to registers from 0 to 254 (yes, 254) in order to reset the card.

; 1 frame = 312 lines = 19656 cycles = 0.02 seconds
; 1 cycle ~= 1 microsecond

; 882 samples = 50th of a second = 0.02
; samples to time: (samples/44100)
; samples to frame: (samples/882)
; clock = $369E99 = 3,579,545
; $113D = 4431

ptr	= $D0

	dw nextln,0	; second word is line number
	db $9E
	if C64=1
	db "2061"
	else
	db "4109"
	endif
	db 0 
nextln	db 0,0
	
start
	jsr detect_ym3812
	bcs ym3812_present
	;
	ldx #$00
text_loop1
	lda not_detected_text_data,x
	beq stop
	jsr $FFD2
	inx
	bne text_loop1
	;
stop	;rts
	;
ym3812_present
menu
	ldx #$00
text_loop2
	lda text_data,x
	beq text_data_done
	jsr $FFD2
	inx
	bne text_loop2
	;
text_data_done
keyboard_loop
	lda #$7F
	jsr check_keyboard
	and #$01				; Query keyboard for "1"
	beq song1_selected
	;
	lda #$7F
	jsr check_keyboard
	and #$08				; Query keyboard for "2"
	beq song2_selected
	;
	lda #$FD
	jsr check_keyboard
	and #$01				; Query keyboard for "3"
	beq song3_selected
	;
	lda #$7F
	jsr check_keyboard
	and #$80				; Query keyboard for "Run/Stop"
	bne keyboard_loop
	;
	sei
	jsr wait_for_key_release
	cli
	;
	rts					; exit
	;
song1_selected
	lda #song1 & 255
	ldx #song1 >> 8
	bne start_playback
	;
song2_selected
	lda #song2 & 255
	ldx #song2 >> 8
	bne start_playback
	;
song3_selected
	lda #song3 & 255
	ldx #song3 >> 8
	;
start_playback
	sei
	;
	; --- set pointer to beginning of data
	;
	sta ptr
	stx ptr+1
	;
	ldx #$00
text_loop3
	lda playing,x
	beq playing_text_done
	jsr $FFD2
	inx
	bne text_loop3
	;
playing_text_done
	;
	jsr reset_ym3812
	;
	sta $FF3F
	;
	; --- player
	;
main_loop
	ldy #$00
read_next_byte
	lda (ptr),y
	cmp #$FF
	beq do_delay
	;
	sta REGNUM				; wait 3.3 microseconds (3 cycles) - iny/lda(zp) takes 7 cycles
	;sta $FF19
	;
	iny
	lda (ptr),y
	sta REGVAL				; wait 23 microseconds (23 cycles)
	;sta $FF19
	;
move_ptr_ahead
	iny					; 2
	tya					; 2
	clc					; 2
	adc ptr					; 3
	sta ptr					; 3
	lda ptr+1				; 3
	adc #$00				; 2
	sta ptr+1				; 3
	;
	jmp main_loop
	;
do_delay
	iny
	lda (ptr),y
	cmp #$FF
	beq vgm_data_end
	;
	tax
delay	lda #$CB
	cmp RASTER
	bne *-3
	cmp RASTER
	beq *-3
	;
	lda #$7F
	jsr check_keyboard
	and #$80				; Query keyboard for "Run/Stop"
	beq stop_playing
	;
	dex
	bne delay
	beq move_ptr_ahead
	;
vgm_data_end
	sta $FF3E
	cli
	jmp menu

check_keyboard
	if C64=1
	sta $DC00
	lda $DC01
	else
	sta $FD30
	sta $FF08
	lda $FF08
	endif
	rts

wait_for_key_release
	lda #$00				; wait for ALL keys to be released (except shift lock)
	jsr check_keyboard
	cmp #$FF
	bne wait_for_key_release
	rts

stop_playing
	sta $FF3E
	jsr reset_ym3812
	jsr wait_for_key_release
	cli
	jmp menu

detect_ym3812
	sei					; interrupts must be disabled during detection
	;
	ldy #$04				; Reset both timers by writing 60h to register 4.
	lda #$60
	jsr set_reg
	;
	ldy #$04				; Enable the interrupts by writing 80h to register 4 (NOTE: thismust be a separate step)
	lda #$80
	jsr set_reg
	;
	lda STATUS
	and #$E0
	bne no_ym3812
	;
	ldy #$02				; Write FFh to register 2 (Timer 1).
	lda #$FF
	jsr set_reg
	;
	ldy #$04				; Start timer 1 by writing 21h to register 4.
	lda #$21
	jsr set_reg
	;
	ldx #40					; delay for (at least) 80 microseconds
	dex
	bpl *-1
	;
	lda STATUS
	and #$E0
	cmp #$C0
	bne no_ym3812
	;
	ldy #$04				; Reset both timers by writing 60h to register 4.
	lda #$60
	jsr set_reg
	;
	cli
	sec					; detected!
	rts
	;
no_ym3812
	ldy #$04				; Reset both timers by writing 60h to register 4.
	lda #$60
	jsr set_reg
	;
	cli
	clc					; not detected
	rts
 
reset_ym3812
	;
	; --- reset card
	;
	ldy #$00
	tya
reset_loop
	jsr set_reg
	;
	iny
	cpy #$FF
	bne reset_loop

set_reg
	sty REGNUM				; wait 3
	nop
	nop
	sta REGVAL				; wait 23
	;
	nop
	nop
	nop
	nop
	;
	nop
	nop
	nop
	nop
	;
	rts

not_detected_text_data
	db "YM3812 NOT DETECTED.",$0D,0

text_data
	db $90,$0D,"YM3812 TECH DEMO",$0D
	db $9A,"BY CSABO/LOD 2023",$0D,$0D
	db $90,"[1] HIGHWAYS",$0D
	db "[2] GARGOYLES",$0D
	db "[3] URBANISM",$0D
	db "[RUN/STOP] EXIT",$0D,$0D,0

playing
	db $90,"PLAYING... [RUN/STOP] TO EXIT",$0D,0

song1
	include "highways.asm"
song2
	include "gargoyles.asm"
song3
	include "urbanism.asm"
;[eof]